@@ -9,7 +9,7 @@ mod worker;
 
 use std::cmp::Ordering;
 use std::fs::{self, File};
-use std::io::{Read, Write};
+use std::io::{Read, Write, BufWriter};
 use std::iter::FromIterator;
 use std::path::Path;
 use std::path::PathBuf;
@@ -18,8 +18,7 @@ use std::sync::Mutex;
 use std::time::Instant;
 
 use morsels_common::tokenize::Tokenizer;
-use morsels_common::MorselsLanguageConfig;
-use morsels_common::DOC_INFO_FILE_NAME;
+use morsels_common::{MorselsLanguageConfig, BITMAP_DOCINFO_DICT_TABLE_FILE, BitmapDocinfoDicttableReader};
 use morsels_lang_ascii::ascii;
 use morsels_lang_latin::latin;
 use morsels_lang_chinese::chinese;
@@ -32,6 +31,7 @@ use crate::fieldinfo::FieldsConfig;
 use crate::loader::csv::CsvLoader;
 use crate::loader::html::HtmlLoader;
 use crate::loader::json::JsonLoader;
+use crate::loader::txt::TxtLoader;
 use crate::loader::Loader;
 use crate::worker::miner::WorkerMiner;
 use crate::worker::{MainToWorkerMessage, Worker, WorkerToMainMessage};
@@ -134,6 +134,7 @@ impl MorselsIndexingConfig {
                 "HtmlLoader" => loaders.push(HtmlLoader::get_new_html_loader(value)),
                 "CsvLoader" => loaders.push(CsvLoader::get_new_csv_loader(value)),
                 "JsonLoader" => loaders.push(JsonLoader::get_new_json_loader(value)),
+                "TxtLoader" => loaders.push(TxtLoader::get_new_txt_loader(value)),
                 _ => panic!("Unknown loader type encountered in config"),
             }
         }
@@ -209,7 +210,6 @@ pub struct Indexer {
     num_workers_writing_blocks: Arc<Mutex<usize>>,
     lang_config: MorselsLanguageConfig,
     is_incremental: bool,
-    delete_unencountered_external_ids: bool,
     start_doc_id: u32,
     start_block_number: u32,
     incremental_info: IncrementalIndexInfo,
@@ -223,17 +223,26 @@ impl Indexer {
         mut is_incremental: bool,
         use_content_hash: bool,
         preserve_output_folder: bool,
-        delete_unencountered_external_ids: bool,
     ) -> Indexer {
         fs::create_dir_all(output_folder_path).expect("could not create output directory!");
 
         let raw_config_normalised = &String::from_iter(normalized(config.raw_config.chars()));
 
-        let incremental_info = IncrementalIndexInfo::new_from_output_folder(
-            &output_folder_path,
+        let bitmap_docinfo_dicttable_path = output_folder_path.join(BITMAP_DOCINFO_DICT_TABLE_FILE);
+        let mut bitmap_docinfo_dicttable_rdr = if let Ok(mut file) = File::open(bitmap_docinfo_dicttable_path) {
+            let mut buf = Vec::new();
+            file.read_to_end(&mut buf).unwrap();
+            Some(BitmapDocinfoDicttableReader { buf, pos: 0 })
+        } else {
+            None
+        };
+
+        let mut incremental_info = IncrementalIndexInfo::new_from_output_folder(
+            output_folder_path,
             raw_config_normalised,
             &mut is_incremental,
-            use_content_hash
+            use_content_hash,
+            bitmap_docinfo_dicttable_rdr.as_mut(),
         );
 
         if !is_incremental && !preserve_output_folder {
@@ -316,18 +325,23 @@ impl Indexer {
             ))
         };
 
-        let doc_infos = Arc::from(Mutex::from(if is_incremental {
-            let mut doc_infos_vec: Vec<u8> = Vec::new();
-            File::open(output_folder_path.join(DOC_INFO_FILE_NAME))
-                .unwrap()
-                .read_to_end(&mut doc_infos_vec)
-                .unwrap();
 
-            DocInfos::from_search_docinfo(doc_infos_vec, field_infos.num_scored_fields)
+        let doc_infos = Arc::from(Mutex::from(if is_incremental {
+            DocInfos::from_search_docinfo(
+                bitmap_docinfo_dicttable_rdr.as_mut().expect("missing docinfo metadata file!"),
+                field_infos.num_scored_fields,
+            )
         } else {
             DocInfos::init_doc_infos(field_infos.num_scored_fields)
         }));
 
+        if is_incremental {
+            incremental_info.setup_dictionary(
+                output_folder_path,
+                bitmap_docinfo_dicttable_rdr.as_mut().expect("missing dicttable metadata file!"),
+            );
+        }
+
         let doc_id_counter = doc_infos.lock().unwrap().doc_lengths.len() as u32;
 
         #[cfg(debug_assertions)]
@@ -405,7 +419,6 @@ impl Indexer {
             num_workers_writing_blocks,
             lang_config: config.lang_config,
             is_incremental,
-            delete_unencountered_external_ids,
             start_doc_id: doc_id_counter,
             start_block_number: 0,
             incremental_info,
@@ -564,7 +577,7 @@ impl Indexer {
 
         self.write_morsels_config();
 
-        self.incremental_info.write(&self.output_folder_path, self.doc_id_counter);
+        self.incremental_info.write_info(&self.output_folder_path);
 
         if !self.is_deletion_only_run() {
             spimireader::common::cleanup_blocks(first_block, last_block, &self.output_folder_path);
@@ -582,11 +595,15 @@ impl Indexer {
     }
 
     fn merge_blocks(&mut self, first_block: u32, last_block: u32) {
+        let bitmap_docinfo_dicttable_file = self.output_folder_path.join(BITMAP_DOCINFO_DICT_TABLE_FILE);
+        let mut bitmap_docinfo_dicttable_writer = BufWriter::new(
+            File::create(bitmap_docinfo_dicttable_file).unwrap()
+        );
+
         let num_blocks = last_block - first_block + 1;
         if self.is_incremental {
-            if self.delete_unencountered_external_ids {
-                self.incremental_info.delete_unencountered_external_ids();
-            }
+            self.incremental_info.delete_unencountered_external_ids();
+            self.incremental_info.write_invalidation_vec(&mut bitmap_docinfo_dicttable_writer, self.doc_id_counter);
 
             spimireader::incremental::modify_blocks(
                 self.is_deletion_only_run(),
@@ -595,21 +612,27 @@ impl Indexer {
                 first_block,
                 last_block,
                 &self.indexing_config,
+                &self.field_infos,
                 std::mem::take(&mut self.doc_infos),
                 &self.tx_main,
                 &self.output_folder_path,
+                bitmap_docinfo_dicttable_writer,
                 &mut self.incremental_info,
             );
         } else {
+            self.incremental_info.write_invalidation_vec(&mut bitmap_docinfo_dicttable_writer, self.doc_id_counter);
+
             spimireader::full::merge_blocks(
                 self.doc_id_counter,
                 num_blocks,
                 first_block,
                 last_block,
                 &self.indexing_config,
+                &self.field_infos,
                 std::mem::take(&mut self.doc_infos),
                 &self.tx_main,
                 &self.output_folder_path,
+                bitmap_docinfo_dicttable_writer,
                 &mut self.incremental_info,
             );
         }
