@@ -1,3 +1,3 @@
 pub mod common;
-pub mod dynamic;
+pub mod incremental;
 pub mod full;
