@@ -1,4 +1,6 @@
 use std::collections::BinaryHeap;
+use std::fs::File;
+use std::io::BufWriter;
 use std::io::Write;
 use std::path::Path;
 use std::sync::Arc;
@@ -6,9 +8,8 @@ use std::sync::Mutex;
 
 use dashmap::DashMap;
 
-use morsels_common::DOC_INFO_FILE_NAME;
-
 use crate::docinfo::DocInfos;
+use crate::fieldinfo::FieldInfos;
 use crate::spimireader::common::{
     self, postings_stream::PostingsStream, terms, PostingsStreamDecoder, TermDocsForMerge,
 };
@@ -26,9 +27,11 @@ pub fn merge_blocks(
     first_block: u32,
     last_block: u32,
     indexing_config: &MorselsIndexingConfig,
+    field_infos: &Arc<FieldInfos>,
     doc_infos: Arc<Mutex<DocInfos>>,
     tx_main: &Sender<MainToWorkerMessage>,
     output_folder_path: &Path,
+    mut docinfo_dicttable_writer: BufWriter<File>,
     incremental_info: &mut IncrementalIndexInfo,
 ) {
     /*
@@ -55,8 +58,11 @@ pub fn merge_blocks(
             .expect("No thread should be holding doc infos arc when merging blocks")
             .into_inner()
             .expect("No thread should be holding doc infos mutex when merging blocks");
-        doc_infos_unwrapped_inner
-            .finalize_and_flush(output_folder_path.join(DOC_INFO_FILE_NAME), doc_id_counter);
+        doc_infos_unwrapped_inner.finalize_and_flush(
+            &mut docinfo_dicttable_writer,
+            doc_id_counter, field_infos.num_scored_fields,
+            incremental_info
+        );
 
         Arc::from(doc_infos_unwrapped_inner)
     };
@@ -78,7 +84,7 @@ pub fn merge_blocks(
     Sort and aggregate worker docIds into one vector
     */
 
-    let (mut dict_table_writer, mut dict_string_writer) = common::get_dict_writers(output_folder_path);
+    let mut dict_string_writer = common::get_dictstring_writer(output_folder_path);
     let mut pl_writer = common::get_pl_writer(output_folder_path, 0, indexing_config.num_pls_per_dir);
 
     // Preallocate some things
@@ -119,7 +125,7 @@ pub fn merge_blocks(
         let start_pl_offset = common::write_new_term_postings(
             &mut curr_combined_term_docs,
             &mut varint_buf,
-            Some(&mut dict_table_writer),
+            Some(&mut docinfo_dicttable_writer),
             &mut curr_pl,
             &mut pl_writer,
             &mut curr_pl_offset,
@@ -137,9 +143,9 @@ pub fn merge_blocks(
         // ---------------------------------------------
         // Dictionary table writing: doc freq (var-int), pl offset (var-int)
 
-        dict_table_writer.write_all(varint::get_var_int(doc_freq, &mut varint_buf)).unwrap();
+        docinfo_dicttable_writer.write_all(varint::get_var_int(doc_freq, &mut varint_buf)).unwrap();
 
-        dict_table_writer
+        docinfo_dicttable_writer
             .write_all(varint::get_var_int(start_pl_offset - prev_pl_start_offset, &mut varint_buf))
             .unwrap();
         prev_pl_start_offset = start_pl_offset;
@@ -156,7 +162,7 @@ pub fn merge_blocks(
 
     pl_writer.flush(curr_pl_offset, indexing_config.pl_cache_threshold, &mut incremental_info.pl_names_to_cache);
 
-    dict_table_writer.flush().unwrap();
+    docinfo_dicttable_writer.flush().unwrap();
     dict_string_writer.flush().unwrap();
 
     incremental_info.last_pl_number = if curr_pl_offset != 0 || curr_pl == 0 {
