@@ -11,6 +11,7 @@ use rustc_hash::FxHashMap;
 use crate::FieldInfo;
 use crate::FieldInfos;
 
+#[derive(Default)]
 pub struct DocField {
     pub field_tf: u32,
     pub positions: Vec<u32>,
@@ -22,12 +23,6 @@ impl Clone for DocField {
     }
 }
 
-impl Default for DocField {
-    fn default() -> Self {
-        DocField { field_tf: 0, positions: Vec::new() }
-    }
-}
-
 pub struct TermDoc {
     pub doc_id: u32,
     pub doc_fields: Vec<DocField>,
